var cluster = require('cluster');
var numberOfCPU = require('os').cpus().length - 1;

function workAround(worker) {
  var listeners = null;

  listeners = worker.process.listeners('exit')[0];
  var exit = listeners[Object.keys(listeners)[0]];

  listeners = worker.process.listeners('disconnect')[0];
  var disconnect = listeners[Object.keys(listeners)[0]];

  worker.process.removeListener('exit', exit);
  worker.process.once('exit', function(exitCode, signalCode) {
    if (worker.state != 'disconnected') {
      disconnect();
    }
    exit(exitCode, signalCode);
  });

  worker.process.on('uncaughtException', function(err) {
    process.kill(process.pid, 'SIGHUP');
  });
};

module.exports = function(spawnFunc, numberOfWorkers, masterFunc) {
  numberOfWorkers = numberOfWorkers || numberOfCPU;

  if (cluster.isMaster) {
    cluster.on('fork', function(worker) {
      workAround(worker);
    });
    if (masterFunc) masterFunc();
    for (var i = 0; i < numberOfWorkers; i++) cluster.fork();  
  } else {
    if (spawnFunc) spawnFunc();
  }
};
